-- Этот файл содержит структурное описание операционного устройства
-- состоящего из управляющего автомата МИЛИ и операционного автомата
-- управление осуществляется на уровне микрокоманд


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library work;
use work.mk_pack.all;			-- подключаем пакет, в котором содержится декларация типа сигнала control_y
	
entity OY_mk is					-- интерфейс операционного устройства
generic(n:integer:=4);			-- параметр n задает разрядность устройства
	PORT
	(
		a		:	 in std_logic_vector(n-1 downto 0);		-- 1-ый операнд (множимое)
		b		:	 in std_logic_vector(n-1 downto 0);		-- 2-ой операнд (множитель)
		clk	:	 in std_logic;									--синхросигнал 
		set	:	 in std_logic;									--сигнал начальной установки
		sno	:	 in std_logic;									--сигнал начала операции
		sko	: 	 out std_logic;								--сигнал конца операции
		rc		:	 out std_logic_vector(2*n-1 downto 0)	-- результат (произведение)
	);
END OY_mk;

architecture struct of OY_mk is --описание архитектуного тела ОУ с использованием структурного стиля
-- В описании архитектуры декларируются две компоненты ОА и УА	

		
component Operation_automat_mk -- декларация компонента ОА
generic(n:integer);											-- параметр n задает разрядность устройства
PORT (y:  in control_y;    								-- управляющие сигналы задают микрокоманды
	   x: out std_logic_vector(2 downto 0);    		-- логические условия
	   a: in std_logic_vector(n-1 downto 0);    		-- 1-ый операнд (множимое)
	   b: in std_logic_vector(n-1 downto 0);    		-- 2-ой операнд (множитель)
	   rc: buffer std_logic_vector(2*n-1 downto 0);	-- результат (произведение)	
	   clk: in std_logic										--синхросигнал 
);
end component;

component ControlUnit_mk -- декларация компонента УА
port (y: out control_y;    							--управляющие сигналы		
	   x: in std_logic_vector(2 downto 0);    	-- логические условия
	   clk: in std_logic;                        --синхросигнал 
		set: in std_logic; 								--сигнал начальной установки
		sno: in std_logic;								--сигнал начала операции
		sko: out std_logic								--сигнал конца операции
		);
		end component;
-- Декларируем сигналы для соединения компонентов		
		
		signal x_X: std_logic_vector(2 downto 0);  		--логические условия
		signal y_Y: control_y;									--микрокоманды
   
begin	-- ниже приводятся экземпляры объектов, каждый экземпляр имеет уникальную метку,играющую роль его имени
unit_OA: Operation_automat_mk		-- unit_OA - это имя ОА
generic map(n => 4)					-- n=4					
port map (y_Y,x_X,a,b,rc,clk);	-- карта портов для ОА
		
unit_YA: ControlUnit_mk				-- unit_YA - это имя УА
port map(y=>y_Y,sko=>sko,x=>x_X,clk=>clk,set=>set,sno=>sno);-- карта портов для УА

end struct;